@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2005-2021 The OpenLDAP Foundation.
+ * Copyright 2005-2022 The OpenLDAP Foundation.
  * Portions copyright 2004-2005 Symas Corporation.
  * All rights reserved.
  *
@@ -753,7 +753,7 @@ accesslog_purge( void *ctx, void *arg )
 			ldap_pvt_thread_mutex_lock( &li->li_log_mutex );
 			mod.sml_numvals = li->li_numcsns;
 			mod.sml_values = li->li_mincsn;
-			mod.sml_nvalues = NULL;
+			mod.sml_nvalues = li->li_mincsn;
 			mod.sml_desc = ad_minCSN;
 			mod.sml_op = LDAP_MOD_REPLACE;
 			mod.sml_flags = SLAP_MOD_INTERNAL;
@@ -926,10 +926,10 @@ log_cf_gen(ConfigArgs *c)
 					ch_free( la );
 				}
 			} else {
-				log_attr *la = NULL, **lp;
+				log_attr *la = li->li_oldattrs, **lp = &li->li_oldattrs;
 				int i;
 
-				for ( lp = &li->li_oldattrs, i=0; i < c->valx; i++ ) {
+				for ( i=0; i < c->valx; i++ ) {
 					la = *lp;
 					lp = &la->next;
 				}
@@ -946,10 +946,10 @@ log_cf_gen(ConfigArgs *c)
 					ch_free( lb );
 				}
 			} else {
-				log_base *lb = NULL, **lp;
+				log_base *lb = li->li_bases, **lp = &li->li_bases;
 				int i;
 
-				for ( lp = &li->li_bases, i=0; i < c->valx; i++ ) {
+				for ( i=0; i < c->valx; i++ ) {
 					lb = *lp;
 					lp = &lb->lb_next;
 				}
@@ -1029,14 +1029,23 @@ log_cf_gen(ConfigArgs *c)
 			int i;
 			AttributeDescription *ad;
 			const char *text;
+			log_attr **lp = &li->li_oldattrs;
+
+			for ( i=0; *lp && ( c->valx < 0 || i < c->valx ); i++ )
+				lp = &(*lp)->next;
 
 			for ( i=1; i< c->argc; i++ ) {
 				ad = NULL;
 				if ( slap_str2ad( c->argv[i], &ad, &text ) == LDAP_SUCCESS ) {
 					log_attr *la = ch_malloc( sizeof( log_attr ));
 					la->attr = ad;
-					la->next = li->li_oldattrs;
-					li->li_oldattrs = la;
+					if ( *lp ) {
+						la->next = (*lp)->next;
+					} else {
+						la->next = NULL;
+					}
+					*lp = la;
+					lp = &la->next;
 				} else {
 					snprintf( c->cr_msg, sizeof( c->cr_msg ), "%s <%s>: %s",
 						c->argv[0], c->argv[i], text );
@@ -1049,7 +1058,13 @@ log_cf_gen(ConfigArgs *c)
 			}
 			break;
 		case LOG_BASE: {
+			int i;
 			slap_mask_t m = 0;
+			log_base **lp = &li->li_bases;
+
+			for ( i=0; *lp && ( c->valx < 0 || i < c->valx ); i++ )
+				lp = &(*lp)->lb_next;
+
 			rc = verbstring_to_mask( logops, c->argv[1], '|', &m );
 			if ( rc == 0 ) {
 				struct berval dn, ndn;
@@ -1072,7 +1087,12 @@ log_cf_gen(ConfigArgs *c)
 					*ptr++ = '"';
 					lb->lb_ops = m;
 					lb->lb_next = li->li_bases;
-					li->li_bases = lb;
+					if ( *lp ) {
+						lb->lb_next = (*lp)->lb_next;
+					} else {
+						lb->lb_next = NULL;
+					}
+					*lp = lb;
 				} else {
 					snprintf( c->cr_msg, sizeof( c->cr_msg ), "%s invalid DN: %s",
 						c->argv[0], c->argv[2] );
@@ -1602,7 +1622,7 @@ accesslog_response(Operation *op, SlapReply *rs)
 						op->o_log_prefix, li->li_db_suffix.bv_val );
 				assert(0);
 			}
-			op2.o_csn = op->o_csn;
+			slap_queue_csn( &op2, &op->o_csn );
 		}
 	}
 
@@ -1943,26 +1963,6 @@ accesslog_response(Operation *op, SlapReply *rs)
 	/* contextCSN updates may still reach here */
 	op2.o_dont_replicate = op->o_dont_replicate;
 
-	if ( lo->mask & LOG_OP_WRITES ) {
-		struct berval maxcsn;
-		char cbuf[LDAP_PVT_CSNSTR_BUFSIZE];
-		int foundit;
-		cbuf[0] = '\0';
-		maxcsn.bv_val = cbuf;
-		maxcsn.bv_len = sizeof(cbuf);
-		/* If there was a commit CSN on the main DB,
-		 * we must propagate it to the log DB for its
-		 * own syncprov. Otherwise, don't generate one.
-		 */
-		slap_get_commit_csn( op, &maxcsn, &foundit );
-		if ( !BER_BVISEMPTY( &maxcsn ) ) {
-			slap_queue_csn( &op2, &op->o_csn );
-		} else {
-			attr_merge_normalize_one( e, slap_schema.si_ad_entryCSN,
-				&op->o_csn, op->o_tmpmemctx );
-		}
-	}
-
 	op2.o_bd->be_add( &op2, &rs2 );
 	if ( rs2.sr_err != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_SYNC, "%s accesslog_response: "
@@ -2041,6 +2041,16 @@ static int
 accesslog_op_misc( Operation *op, SlapReply *rs )
 {
 	slap_callback *sc;
+	slap_verbmasks *lo;
+	int logop;
+
+	logop = accesslog_op2logop( op );
+	lo = logops+logop+EN_OFFSET;
+
+	/* ignore these internal reads */
+	if (( lo->mask & LOG_OP_READS ) && op->o_do_not_cache ) {
+		return SLAP_CB_CONTINUE;
+	}
 
 	sc = op->o_tmpcalloc( 1, sizeof(slap_callback), op->o_tmpmemctx );
 	sc->sc_response = accesslog_response;
@@ -2075,11 +2085,6 @@ accesslog_op_mod( Operation *op, SlapReply *rs )
 	logop = accesslog_op2logop( op );
 	lo = logops+logop+EN_OFFSET;
 
-	/* Ignore these internal reads */
-	if (( lo->mask & LOG_OP_READS ) && op->o_do_not_cache ) {
-		return SLAP_CB_CONTINUE;
-	}
-
 	if ( !( li->li_ops & lo->mask )) {
 		log_base *lb;
 		int i = 0;
@@ -2166,6 +2171,12 @@ accesslog_unbind( Operation *op, SlapReply *rs )
 			return SLAP_CB_CONTINUE;
 	}
 
+	op2.o_hdr = op->o_hdr;
+	op2.o_tag = LDAP_REQ_ADD;
+	op2.o_bd = li->li_db;
+	op2.o_csn.bv_val = csnbuf;
+	op2.o_csn.bv_len = sizeof(csnbuf);
+
 	ldap_pvt_thread_mutex_lock( &li->li_op_rmutex );
 
 	if ( SLAP_LASTMOD( li->li_db ) ) {
@@ -2174,8 +2185,6 @@ accesslog_unbind( Operation *op, SlapReply *rs )
 		 * ordering
 		 */
 		if ( BER_BVISEMPTY( &op->o_csn ) ) {
-			op2.o_csn.bv_val = csnbuf;
-			op2.o_csn.bv_len = sizeof(csnbuf);
 			slap_get_csn( &op2, &op2.o_csn, 1 );
 		} else {
 			Debug( LDAP_DEBUG_ANY, "%s accesslog_unbind: "
@@ -2190,9 +2199,6 @@ accesslog_unbind( Operation *op, SlapReply *rs )
 	ldap_pvt_thread_mutex_unlock( &li->li_op_rmutex );
 
 	e = accesslog_entry( op, rs, li, LOG_EN_UNBIND, &op2 );
-	op2.o_hdr = op->o_hdr;
-	op2.o_tag = LDAP_REQ_ADD;
-	op2.o_bd = li->li_db;
 	op2.o_dn = li->li_db->be_rootdn;
 	op2.o_ndn = li->li_db->be_rootndn;
 	op2.o_req_dn = e->e_name;
@@ -2245,6 +2251,12 @@ accesslog_abandon( Operation *op, SlapReply *rs )
 			return SLAP_CB_CONTINUE;
 	}
 
+	op2.o_hdr = op->o_hdr;
+	op2.o_tag = LDAP_REQ_ADD;
+	op2.o_bd = li->li_db;
+	op2.o_csn.bv_val = csnbuf;
+	op2.o_csn.bv_len = sizeof(csnbuf);
+
 	ldap_pvt_thread_mutex_lock( &li->li_op_rmutex );
 	if ( SLAP_LASTMOD( li->li_db ) ) {
 		/*
@@ -2252,8 +2264,6 @@ accesslog_abandon( Operation *op, SlapReply *rs )
 		 * ordering
 		 */
 		if ( BER_BVISEMPTY( &op->o_csn ) ) {
-			op2.o_csn.bv_val = csnbuf;
-			op2.o_csn.bv_len = sizeof(csnbuf);
 			slap_get_csn( &op2, &op2.o_csn, 1 );
 		} else {
 			Debug( LDAP_DEBUG_ANY, "%s accesslog_abandon: "
@@ -2274,9 +2284,6 @@ accesslog_abandon( Operation *op, SlapReply *rs )
 		attr_merge_one( e, ad_reqId, &bv, NULL );
 	} /* else? */
 
-	op2.o_hdr = op->o_hdr;
-	op2.o_tag = LDAP_REQ_ADD;
-	op2.o_bd = li->li_db;
 	op2.o_dn = li->li_db->be_rootdn;
 	op2.o_ndn = li->li_db->be_rootndn;
 	op2.o_req_dn = e->e_name;
